@@ -1,6 +1,9 @@
 #ifndef _SWAY_IPC_CLIENT_H
 #define _SWAY_IPC_CLIENT_H
 
+// arbitrary number, it's probably sufficient, higher number = more memory usage
+#define JSON_MAX_DEPTH 512
+
 #include <stdbool.h>
 #include <stdint.h>
 #include <sys/time.h>
