@@ -2,8 +2,15 @@
 #define _SWAY_STRINGOP_H
 
 #include <stdbool.h>
+#include <stddef.h>
 #include "list.h"
 
+#ifdef __GNUC__
+#define _SWAY_ATTRIB_PRINTF(start, end) __attribute__((format(printf, start, end)))
+#else
+#define _SWAY_ATTRIB_PRINTF(start, end)
+#endif
+
 void strip_whitespace(char *str);
 void strip_quotes(char *str);
 
@@ -30,4 +37,7 @@ char *argsep(char **stringp, const char *delim, char *matched_delim);
 // Expand a path using shell replacements such as $HOME and ~
 bool expand_path(char **path);
 
+char *vformat_str(const char *fmt, va_list args) _SWAY_ATTRIB_PRINTF(1, 0);
+char *format_str(const char *fmt, ...) _SWAY_ATTRIB_PRINTF(1, 2);
+
 #endif
