@@ -1,8 +1,6 @@
 #ifndef _SWAY_DESKTOP_IDLE_INHIBIT_V1_H
 #define _SWAY_DESKTOP_IDLE_INHIBIT_V1_H
 #include <wlr/types/wlr_idle_inhibit_v1.h>
-#include <wlr/types/wlr_idle.h>
-#include "sway/server.h"
 
 enum sway_idle_inhibit_mode {
 	INHIBIT_IDLE_APPLICATION,  // Application set inhibitor (when visible)
@@ -16,12 +14,9 @@ struct sway_idle_inhibit_manager_v1 {
 	struct wlr_idle_inhibit_manager_v1 *wlr_manager;
 	struct wl_listener new_idle_inhibitor_v1;
 	struct wl_list inhibitors;
-
-	struct wlr_idle *idle;
 };
 
 struct sway_idle_inhibitor_v1 {
-	struct sway_idle_inhibit_manager_v1 *manager;
 	struct wlr_idle_inhibitor_v1 *wlr_inhibitor;
 	struct sway_view *view;
 	enum sway_idle_inhibit_mode mode;
@@ -33,8 +28,7 @@ struct sway_idle_inhibitor_v1 {
 bool sway_idle_inhibit_v1_is_active(
 	struct sway_idle_inhibitor_v1 *inhibitor);
 
-void sway_idle_inhibit_v1_check_active(
-	struct sway_idle_inhibit_manager_v1 *manager);
+void sway_idle_inhibit_v1_check_active(void);
 
 void sway_idle_inhibit_v1_user_inhibitor_register(struct sway_view *view,
 		enum sway_idle_inhibit_mode mode);
@@ -48,6 +42,6 @@ struct sway_idle_inhibitor_v1 *sway_idle_inhibit_v1_application_inhibitor_for_vi
 void sway_idle_inhibit_v1_user_inhibitor_destroy(
 		struct sway_idle_inhibitor_v1 *inhibitor);
 
-struct sway_idle_inhibit_manager_v1 *sway_idle_inhibit_manager_v1_create(
-	struct wl_display *wl_display, struct wlr_idle *idle);
+bool sway_idle_inhibit_manager_v1_init(void);
+
 #endif
