@@ -1,4 +1,3 @@
-#define _POSIX_C_SOURCE 200809L
 #include <string.h>
 #include "sway/commands.h"
 #include "sway/config.h"
@@ -10,8 +9,8 @@
 #include "stringop.h"
 #include "util.h"
 
-static void rebuild_marks_iterator(struct sway_container *con, void *data) {
-	container_update_marks_textures(con);
+static void title_bar_update_iterator(struct sway_container *con, void *data) {
+	container_update_marks(con);
 }
 
 struct cmd_results *cmd_show_marks(int argc, char **argv) {
@@ -23,12 +22,7 @@ struct cmd_results *cmd_show_marks(int argc, char **argv) {
 	config->show_marks = parse_boolean(argv[0], config->show_marks);
 
 	if (config->show_marks) {
-		root_for_each_container(rebuild_marks_iterator, NULL);
-	}
-
-	for (int i = 0; i < root->outputs->length; ++i) {
-		struct sway_output *output = root->outputs->items[i];
-		output_damage_whole(output);
+		root_for_each_container(title_bar_update_iterator, NULL);
 	}
 
 	return cmd_results_new(CMD_SUCCESS, NULL);
