@@ -1,12 +1,11 @@
-#define _POSIX_C_SOURCE 200809L
 #include <ctype.h>
 #include <stdio.h>
 #include <string.h>
 #include <math.h>
+#include <assert.h>
 #include <wlr/config.h>
 #include <wlr/types/wlr_cursor.h>
 #include <wlr/types/wlr_keyboard_group.h>
-#include <wlr/types/wlr_input_inhibitor.h>
 #include <wlr/types/wlr_virtual_keyboard_v1.h>
 #include <wlr/types/wlr_virtual_pointer_v1.h>
 #include "sway/config.h"
@@ -67,8 +66,15 @@ struct sway_seat *input_manager_sway_seat_from_wlr_seat(struct wlr_seat *wlr_sea
 }
 
 char *input_device_get_identifier(struct wlr_input_device *device) {
-	int vendor = device->vendor;
-	int product = device->product;
+	int vendor = 0, product = 0;
+#if WLR_HAS_LIBINPUT_BACKEND
+	if (wlr_input_device_is_libinput(device)) {
+		struct libinput_device *libinput_dev = wlr_libinput_get_device_handle(device);
+		vendor = libinput_device_get_id_vendor(libinput_dev);
+		product = libinput_device_get_id_product(libinput_dev);
+	}
+#endif
+
 	char *name = strdup(device->name ? device->name : "");
 	strip_whitespace(name);
 
@@ -113,7 +119,7 @@ const char *input_device_get_type(struct sway_input_device *device) {
 		return "keyboard";
 	case WLR_INPUT_DEVICE_TOUCH:
 		return "touch";
-	case WLR_INPUT_DEVICE_TABLET_TOOL:
+	case WLR_INPUT_DEVICE_TABLET:
 		return "tablet_tool";
 	case WLR_INPUT_DEVICE_TABLET_PAD:
 		return "tablet_pad";
@@ -284,34 +290,6 @@ static void handle_new_input(struct wl_listener *listener, void *data) {
 	}
 }
 
-static void handle_inhibit_activate(struct wl_listener *listener, void *data) {
-	struct sway_input_manager *input_manager = wl_container_of(
-			listener, input_manager, inhibit_activate);
-	struct sway_seat *seat;
-	wl_list_for_each(seat, &input_manager->seats, link) {
-		seat_set_exclusive_client(seat, input_manager->inhibit->active_client);
-	}
-}
-
-static void handle_inhibit_deactivate(struct wl_listener *listener, void *data) {
-	struct sway_input_manager *input_manager = wl_container_of(
-			listener, input_manager, inhibit_deactivate);
-	struct sway_seat *seat;
-	if (server.session_lock.locked) {
-		// Don't deactivate the grab of a screenlocker
-		return;
-	}
-	wl_list_for_each(seat, &input_manager->seats, link) {
-		seat_set_exclusive_client(seat, NULL);
-		struct sway_node *previous = seat_get_focus(seat);
-		if (previous) {
-			// Hack to get seat to re-focus the return value of get_focus
-			seat_set_focus(seat, NULL);
-			seat_set_focus(seat, previous);
-		}
-	}
-}
-
 static void handle_keyboard_shortcuts_inhibitor_destroy(
 		struct wl_listener *listener, void *data) {
 	struct sway_keyboard_shortcuts_inhibitor *sway_inhibitor =
@@ -454,6 +432,20 @@ void handle_virtual_pointer(struct wl_listener *listener, void *data) {
 	}
 }
 
+static void handle_transient_seat_manager_create_seat(
+		struct wl_listener *listener, void *data) {
+	struct wlr_transient_seat_v1 *transient_seat = data;
+	static uint64_t i;
+	char name[256];
+	snprintf(name, sizeof(name), "transient-%"PRIx64, i++);
+	struct sway_seat *seat = seat_create(name);
+	if (seat && seat->wlr_seat) {
+		wlr_transient_seat_v1_ready(transient_seat, seat->wlr_seat);
+	} else {
+		wlr_transient_seat_v1_deny(transient_seat);
+	}
+}
+
 struct sway_input_manager *input_manager_create(struct sway_server *server) {
 	struct sway_input_manager *input =
 		calloc(1, sizeof(struct sway_input_manager));
@@ -480,14 +472,6 @@ struct sway_input_manager *input_manager_create(struct sway_server *server) {
 		&input->virtual_pointer_new);
 	input->virtual_pointer_new.notify = handle_virtual_pointer;
 
-	input->inhibit = wlr_input_inhibit_manager_create(server->wl_display);
-	input->inhibit_activate.notify = handle_inhibit_activate;
-	wl_signal_add(&input->inhibit->events.activate,
-			&input->inhibit_activate);
-	input->inhibit_deactivate.notify = handle_inhibit_deactivate;
-	wl_signal_add(&input->inhibit->events.deactivate,
-			&input->inhibit_deactivate);
-
 	input->keyboard_shortcuts_inhibit =
 		wlr_keyboard_shortcuts_inhibit_v1_create(server->wl_display);
 	input->keyboard_shortcuts_inhibit_new_inhibitor.notify =
@@ -497,6 +481,15 @@ struct sway_input_manager *input_manager_create(struct sway_server *server) {
 
 	input->pointer_gestures = wlr_pointer_gestures_v1_create(server->wl_display);
 
+	input->transient_seat_manager =
+		wlr_transient_seat_manager_v1_create(server->wl_display);
+	assert(input->transient_seat_manager);
+
+	input->transient_seat_create.notify =
+		handle_transient_seat_manager_create_seat;
+	wl_signal_add(&input->transient_seat_manager->events.create_seat,
+			&input->transient_seat_create);
+
 	return input;
 }
 
