@@ -1,5 +1,6 @@
 #include "sway/config.h"
 #include "sway/input/switch.h"
+#include "sway/server.h"
 #include "log.h"
 
 struct sway_switch *sway_switch_create(struct sway_seat *seat,
@@ -33,9 +34,8 @@ static bool sway_switch_trigger_test(enum sway_switch_trigger trigger,
 }
 
 static void execute_binding(struct sway_switch *sway_switch) {
-	struct sway_seat* seat = sway_switch->seat_device->sway_seat;
-	bool input_inhibited = seat->exclusive_client != NULL ||
-		server.session_lock.locked;
+	struct sway_seat *seat = sway_switch->seat_device->sway_seat;
+	bool locked = server.session_lock.lock;
 
 	list_t *bindings = config->current_mode->switch_bindings;
 	struct sway_switch_binding *matched_binding = NULL;
@@ -52,13 +52,13 @@ static void execute_binding(struct sway_switch *sway_switch) {
 			continue;
 		}
 		bool binding_locked = binding->flags & BINDING_LOCKED;
-		if (!binding_locked && input_inhibited) {
+		if (!binding_locked && locked) {
 			continue;
 		}
 
 		matched_binding = binding;
 
-		if (binding_locked == input_inhibited) {
+		if (binding_locked == locked) {
 			break;
 		}
 	}
