@@ -1,22 +1,23 @@
-#define _POSIX_C_SOURCE 200809L
 #include <stdlib.h>
 #include <strings.h>
 #include <wayland-server-core.h>
+#include <wlr/config.h>
 #include <wlr/render/wlr_renderer.h>
 #include <wlr/types/wlr_buffer.h>
+#include <wlr/types/wlr_ext_foreign_toplevel_list_v1.h>
+#include <wlr/types/wlr_foreign_toplevel_management_v1.h>
+#include <wlr/types/wlr_fractional_scale_v1.h>
 #include <wlr/types/wlr_output_layout.h>
 #include <wlr/types/wlr_server_decoration.h>
 #include <wlr/types/wlr_subcompositor.h>
 #include <wlr/types/wlr_xdg_decoration_v1.h>
-#include "config.h"
-#if HAVE_XWAYLAND
+#if WLR_HAS_XWAYLAND
 #include <wlr/xwayland.h>
 #endif
 #include "list.h"
 #include "log.h"
 #include "sway/criteria.h"
 #include "sway/commands.h"
-#include "sway/desktop.h"
 #include "sway/desktop/transaction.h"
 #include "sway/desktop/idle_inhibit_v1.h"
 #include "sway/desktop/launcher.h"
@@ -24,26 +25,41 @@
 #include "sway/ipc-server.h"
 #include "sway/output.h"
 #include "sway/input/seat.h"
+#include "sway/scene_descriptor.h"
 #include "sway/server.h"
-#include "sway/surface.h"
+#include "sway/sway_text_node.h"
 #include "sway/tree/arrange.h"
 #include "sway/tree/container.h"
 #include "sway/tree/view.h"
 #include "sway/tree/workspace.h"
 #include "sway/config.h"
 #include "sway/xdg_decoration.h"
-#include "pango.h"
 #include "stringop.h"
 
-void view_init(struct sway_view *view, enum sway_view_type type,
+bool view_init(struct sway_view *view, enum sway_view_type type,
 		const struct sway_view_impl *impl) {
+	bool failed = false;
+	view->scene_tree = alloc_scene_tree(root->staging, &failed);
+	view->content_tree = alloc_scene_tree(view->scene_tree, &failed);
+
+	if (!failed && !scene_descriptor_assign(&view->scene_tree->node,
+			SWAY_SCENE_DESC_VIEW, view)) {
+		failed = true;
+	}
+
+	if (failed) {
+		wlr_scene_node_destroy(&view->scene_tree->node);
+		return false;
+	}
+
 	view->type = type;
 	view->impl = impl;
 	view->executed_criteria = create_list();
-	wl_list_init(&view->saved_buffers);
 	view->allow_request_urgent = true;
 	view->shortcuts_inhibit = SHORTCUTS_INHIBIT_DEFAULT;
+	view->tearing_mode = TEARING_WINDOW_HINT;
 	wl_signal_init(&view->events.unmap);
+	return true;
 }
 
 void view_destroy(struct sway_view *view) {
@@ -60,15 +76,10 @@ void view_destroy(struct sway_view *view) {
 		return;
 	}
 	wl_list_remove(&view->events.unmap.listener_list);
-	if (!wl_list_empty(&view->saved_buffers)) {
-		view_remove_saved_buffer(view);
-	}
 	list_free(view->executed_criteria);
 
 	view_assign_ctx(view, NULL);
-
-	free(view->title_format);
-
+	wlr_scene_node_destroy(&view->scene_tree->node);
 	if (view->impl->destroy) {
 		view->impl->destroy(view);
 	} else {
@@ -114,7 +125,7 @@ const char *view_get_instance(struct sway_view *view) {
 	}
 	return NULL;
 }
-#if HAVE_XWAYLAND
+#if WLR_HAS_XWAYLAND
 uint32_t view_get_x11_window_id(struct sway_view *view) {
 	if (view->impl->get_int_prop) {
 		return view->impl->get_int_prop(view, VIEW_PROP_X11_WINDOW_ID);
@@ -147,7 +158,7 @@ const char *view_get_shell(struct sway_view *view) {
 	switch(view->type) {
 	case SWAY_VIEW_XDG_SHELL:
 		return "xdg_shell";
-#if HAVE_XWAYLAND
+#if WLR_HAS_XWAYLAND
 	case SWAY_VIEW_XWAYLAND:
 		return "xwayland";
 #endif
@@ -161,9 +172,9 @@ void view_get_constraints(struct sway_view *view, double *min_width,
 		view->impl->get_constraints(view,
 				min_width, max_width, min_height, max_height);
 	} else {
-		*min_width = DBL_MIN;
+		*min_width = 1;
 		*max_width = DBL_MAX;
-		*min_height = DBL_MIN;
+		*min_height = 1;
 		*max_height = DBL_MAX;
 	}
 }
@@ -353,8 +364,8 @@ void view_autoconfigure(struct sway_view *view) {
 
 	con->pending.content_x = x;
 	con->pending.content_y = y;
-	con->pending.content_width = width;
-	con->pending.content_height = height;
+	con->pending.content_width = fmax(width, 1);
+	con->pending.content_height = fmax(height, 1);
 }
 
 void view_set_activated(struct sway_view *view, bool activated) {
@@ -449,52 +460,6 @@ void view_close_popups(struct sway_view *view) {
 	}
 }
 
-void view_damage_from(struct sway_view *view) {
-	for (int i = 0; i < root->outputs->length; ++i) {
-		struct sway_output *output = root->outputs->items[i];
-		output_damage_from_view(output, view);
-	}
-}
-
-void view_for_each_surface(struct sway_view *view,
-		wlr_surface_iterator_func_t iterator, void *user_data) {
-	if (!view->surface) {
-		return;
-	}
-	if (view->impl->for_each_surface) {
-		view->impl->for_each_surface(view, iterator, user_data);
-	} else {
-		wlr_surface_for_each_surface(view->surface, iterator, user_data);
-	}
-}
-
-void view_for_each_popup_surface(struct sway_view *view,
-		wlr_surface_iterator_func_t iterator, void *user_data) {
-	if (!view->surface) {
-		return;
-	}
-	if (view->impl->for_each_popup_surface) {
-		view->impl->for_each_popup_surface(view, iterator, user_data);
-	}
-}
-
-static void view_subsurface_create(struct sway_view *view,
-	struct wlr_subsurface *subsurface);
-
-static void view_init_subsurfaces(struct sway_view *view,
-	struct wlr_surface *surface);
-
-static void view_child_init_subsurfaces(struct sway_view_child *view_child,
-	struct wlr_surface *surface);
-
-static void view_handle_surface_new_subsurface(struct wl_listener *listener,
-		void *data) {
-	struct sway_view *view =
-		wl_container_of(listener, view, surface_new_subsurface);
-	struct wlr_subsurface *subsurface = data;
-	view_subsurface_create(view, subsurface);
-}
-
 static bool view_has_executed_criteria(struct sway_view *view,
 		struct criteria *criteria) {
 	for (int i = 0; i < view->executed_criteria->length; ++i) {
@@ -533,7 +498,7 @@ void view_execute_criteria(struct sway_view *view) {
 static void view_populate_pid(struct sway_view *view) {
 	pid_t pid;
 	switch (view->type) {
-#if HAVE_XWAYLAND
+#if WLR_HAS_XWAYLAND
 	case SWAY_VIEW_XWAYLAND:;
 		struct wlr_xwayland_surface *surf =
 			wlr_xwayland_surface_try_from_wlr_surface(view->surface);
@@ -623,6 +588,14 @@ static struct sway_workspace *select_workspace(struct sway_view *view) {
 	return NULL;
 }
 
+static void update_ext_foreign_toplevel(struct sway_view *view) {
+	struct wlr_ext_foreign_toplevel_handle_v1_state toplevel_state = {
+		.app_id = view_get_app_id(view),
+		.title = view_get_title(view),
+	};
+	wlr_ext_foreign_toplevel_handle_v1_update_state(view->ext_foreign_toplevel, &toplevel_state);
+}
+
 static bool should_focus(struct sway_view *view) {
 	struct sway_seat *seat = input_manager_current_seat();
 	struct sway_container *prev_con = seat_get_focused_container(seat);
@@ -767,6 +740,14 @@ void view_map(struct sway_view *view, struct wlr_surface *wlr_surface,
 		ws = select_workspace(view);
 	}
 
+	if (ws && ws->output) {
+		// Once the output is determined, we can notify the client early about
+		// scale to reduce startup jitter.
+		float scale = ws->output->wlr_output->scale;
+		wlr_fractional_scale_v1_notify_scale(wlr_surface, scale);
+		wlr_surface_set_preferred_buffer_scale(wlr_surface, ceil(scale));
+	}
+
 	struct sway_seat *seat = input_manager_current_seat();
 	struct sway_node *node =
 		seat_get_focus_inactive(seat, ws ? &ws->node : &root->node);
@@ -792,6 +773,13 @@ void view_map(struct sway_view *view, struct wlr_surface *wlr_surface,
 		}
 	}
 
+	struct wlr_ext_foreign_toplevel_handle_v1_state foreign_toplevel_state = {
+		.app_id = view_get_app_id(view),
+		.title = view_get_title(view),
+	};
+	view->ext_foreign_toplevel =
+		wlr_ext_foreign_toplevel_handle_v1_create(server.foreign_toplevel_list, &foreign_toplevel_state);
+
 	view->foreign_toplevel =
 		wlr_foreign_toplevel_handle_v1_create(server.foreign_toplevel_manager);
 	view->foreign_activate_request.notify = handle_foreign_activate_request;
@@ -815,11 +803,6 @@ void view_map(struct sway_view *view, struct wlr_surface *wlr_surface,
 	}
 	ipc_event_window(view->container, "new");
 
-	view_init_subsurfaces(view, wlr_surface);
-	wl_signal_add(&wlr_surface->events.new_subsurface,
-			&view->surface_new_subsurface);
-	view->surface_new_subsurface.notify = view_handle_surface_new_subsurface;
-
 	if (decoration) {
 		view_update_csd_from_client(view, decoration);
 	}
@@ -862,7 +845,7 @@ void view_map(struct sway_view *view, struct wlr_surface *wlr_surface,
 
 	bool set_focus = should_focus(view);
 
-#if HAVE_XWAYLAND
+#if WLR_HAS_XWAYLAND
 	struct wlr_xwayland_surface *xsurface;
 	if ((xsurface = wlr_xwayland_surface_try_from_wlr_surface(wlr_surface))) {
 		set_focus &= wlr_xwayland_icccm_input_model(xsurface) !=
@@ -874,6 +857,10 @@ void view_map(struct sway_view *view, struct wlr_surface *wlr_surface,
 		input_manager_set_focus(&view->container->node);
 	}
 
+	if (view->ext_foreign_toplevel) {
+		update_ext_foreign_toplevel(view);
+	}
+
 	const char *app_id;
 	const char *class;
 	if ((app_id = view_get_app_id(view)) != NULL) {
@@ -886,8 +873,6 @@ void view_map(struct sway_view *view, struct wlr_surface *wlr_surface,
 void view_unmap(struct sway_view *view) {
 	wl_signal_emit_mutable(&view->events.unmap, view);
 
-	wl_list_remove(&view->surface_new_subsurface.link);
-
 	view->executed_criteria->length = 0;
 
 	if (view->urgent_timer) {
@@ -895,6 +880,11 @@ void view_unmap(struct sway_view *view) {
 		view->urgent_timer = NULL;
 	}
 
+	if (view->ext_foreign_toplevel) {
+		wlr_ext_foreign_toplevel_handle_v1_destroy(view->ext_foreign_toplevel);
+		view->ext_foreign_toplevel = NULL;
+	}
+
 	if (view->foreign_toplevel) {
 		wlr_foreign_toplevel_handle_v1_destroy(view->foreign_toplevel);
 		view->foreign_toplevel = NULL;
@@ -941,260 +931,35 @@ void view_update_size(struct sway_view *view) {
 	container_set_geometry_from_content(con);
 }
 
-void view_center_surface(struct sway_view *view) {
+void view_center_and_clip_surface(struct sway_view *view) {
 	struct sway_container *con = view->container;
-	// We always center the current coordinates rather than the next, as the
-	// geometry immediately affects the currently active rendering.
-	con->surface_x = fmax(con->current.content_x, con->current.content_x +
-			(con->current.content_width - view->geometry.width) / 2);
-	con->surface_y = fmax(con->current.content_y, con->current.content_y +
-			(con->current.content_height - view->geometry.height) / 2);
-}
 
-static const struct sway_view_child_impl subsurface_impl;
+	bool clip_to_geometry = true;
 
-static void subsurface_get_view_coords(struct sway_view_child *child,
-		int *sx, int *sy) {
-	struct wlr_surface *surface = child->surface;
-	if (child->parent && child->parent->impl &&
-			child->parent->impl->get_view_coords) {
-		child->parent->impl->get_view_coords(child->parent, sx, sy);
+	if (container_is_floating(con)) {
+		// We always center the current coordinates rather than the next, as the
+		// geometry immediately affects the currently active rendering.
+		int x = (int) fmax(0, (con->current.content_width - view->geometry.width) / 2);
+		int y = (int) fmax(0, (con->current.content_height - view->geometry.height) / 2);
+		clip_to_geometry = !view->using_csd;
+
+		wlr_scene_node_set_position(&view->content_tree->node, x, y);
 	} else {
-		*sx = *sy = 0;
+		wlr_scene_node_set_position(&view->content_tree->node, 0, 0);
 	}
-	struct wlr_subsurface *subsurface =
-		wlr_subsurface_try_from_wlr_surface(surface);
-	*sx += subsurface->current.x;
-	*sy += subsurface->current.y;
-}
 
-static void subsurface_destroy(struct sway_view_child *child) {
-	if (!sway_assert(child->impl == &subsurface_impl,
-			"Expected a subsurface")) {
-		return;
-	}
-	struct sway_subsurface *subsurface = (struct sway_subsurface *)child;
-	wl_list_remove(&subsurface->destroy.link);
-	free(subsurface);
-}
-
-static const struct sway_view_child_impl subsurface_impl = {
-	.get_view_coords = subsurface_get_view_coords,
-	.destroy = subsurface_destroy,
-};
-
-static void subsurface_handle_destroy(struct wl_listener *listener,
-		void *data) {
-	struct sway_subsurface *subsurface =
-		wl_container_of(listener, subsurface, destroy);
-	struct sway_view_child *child = &subsurface->child;
-	view_child_destroy(child);
-}
-
-static void view_child_damage(struct sway_view_child *child, bool whole);
-
-static void view_subsurface_create(struct sway_view *view,
-		struct wlr_subsurface *wlr_subsurface) {
-	struct sway_subsurface *subsurface =
-		calloc(1, sizeof(struct sway_subsurface));
-	if (subsurface == NULL) {
-		sway_log(SWAY_ERROR, "Allocation failed");
-		return;
-	}
-	view_child_init(&subsurface->child, &subsurface_impl, view,
-		wlr_subsurface->surface);
-
-	wl_signal_add(&wlr_subsurface->events.destroy, &subsurface->destroy);
-	subsurface->destroy.notify = subsurface_handle_destroy;
-
-	subsurface->child.mapped = true;
-
-	view_child_damage(&subsurface->child, true);
-}
-
-static void view_child_subsurface_create(struct sway_view_child *child,
-		struct wlr_subsurface *wlr_subsurface) {
-	struct sway_subsurface *subsurface =
-		calloc(1, sizeof(struct sway_subsurface));
-	if (subsurface == NULL) {
-		sway_log(SWAY_ERROR, "Allocation failed");
-		return;
-	}
-	subsurface->child.parent = child;
-	wl_list_insert(&child->children, &subsurface->child.link);
-	view_child_init(&subsurface->child, &subsurface_impl, child->view,
-		wlr_subsurface->surface);
-
-	wl_signal_add(&wlr_subsurface->events.destroy, &subsurface->destroy);
-	subsurface->destroy.notify = subsurface_handle_destroy;
-
-	subsurface->child.mapped = true;
-
-	view_child_damage(&subsurface->child, true);
-}
-
-static bool view_child_is_mapped(struct sway_view_child *child) {
-	while (child) {
-		if (!child->mapped) {
-			return false;
+	// only make sure to clip the content if there is content to clip
+	if (!wl_list_empty(&con->view->content_tree->children)) {
+		struct wlr_box clip = {0};
+		if (clip_to_geometry) {
+			clip = (struct wlr_box){
+				.x = con->view->geometry.x,
+				.y = con->view->geometry.y,
+				.width = con->current.content_width,
+				.height = con->current.content_height,
+			};
 		}
-		child = child->parent;
-	}
-	return true;
-}
-
-static void view_child_damage(struct sway_view_child *child, bool whole) {
-	if (!child || !view_child_is_mapped(child) || !child->view || !child->view->container) {
-		return;
-	}
-	int sx, sy;
-	child->impl->get_view_coords(child, &sx, &sy);
-	desktop_damage_surface(child->surface,
-			child->view->container->pending.content_x -
-				child->view->geometry.x + sx,
-			child->view->container->pending.content_y -
-				child->view->geometry.y + sy, whole);
-}
-
-static void view_child_handle_surface_commit(struct wl_listener *listener,
-		void *data) {
-	struct sway_view_child *child =
-		wl_container_of(listener, child, surface_commit);
-	view_child_damage(child, false);
-}
-
-static void view_child_handle_surface_new_subsurface(
-		struct wl_listener *listener, void *data) {
-	struct sway_view_child *child =
-		wl_container_of(listener, child, surface_new_subsurface);
-	struct wlr_subsurface *subsurface = data;
-	view_child_subsurface_create(child, subsurface);
-}
-
-static void view_child_handle_surface_destroy(struct wl_listener *listener,
-		void *data) {
-	struct sway_view_child *child =
-		wl_container_of(listener, child, surface_destroy);
-	view_child_destroy(child);
-}
-
-static void view_init_subsurfaces(struct sway_view *view,
-		struct wlr_surface *surface) {
-	struct wlr_subsurface *subsurface;
-	wl_list_for_each(subsurface, &surface->current.subsurfaces_below,
-			current.link) {
-		view_subsurface_create(view, subsurface);
-	}
-	wl_list_for_each(subsurface, &surface->current.subsurfaces_above,
-			current.link) {
-		view_subsurface_create(view, subsurface);
-	}
-}
-
-static void view_child_init_subsurfaces(struct sway_view_child *view_child,
-		struct wlr_surface *surface) {
-	struct wlr_subsurface *subsurface;
-	wl_list_for_each(subsurface, &surface->current.subsurfaces_below,
-			current.link) {
-		view_child_subsurface_create(view_child, subsurface);
-	}
-	wl_list_for_each(subsurface, &surface->current.subsurfaces_above,
-			current.link) {
-		view_child_subsurface_create(view_child, subsurface);
-	}
-}
-
-static void view_child_handle_surface_map(struct wl_listener *listener,
-		void *data) {
-	struct sway_view_child *child =
-		wl_container_of(listener, child, surface_map);
-	child->mapped = true;
-	view_child_damage(child, true);
-}
-
-static void view_child_handle_surface_unmap(struct wl_listener *listener,
-		void *data) {
-	struct sway_view_child *child =
-		wl_container_of(listener, child, surface_unmap);
-	view_child_damage(child, true);
-	child->mapped = false;
-}
-
-static void view_child_handle_view_unmap(struct wl_listener *listener,
-		void *data) {
-	struct sway_view_child *child =
-		wl_container_of(listener, child, view_unmap);
-	view_child_damage(child, true);
-	child->mapped = false;
-}
-
-void view_child_init(struct sway_view_child *child,
-		const struct sway_view_child_impl *impl, struct sway_view *view,
-		struct wlr_surface *surface) {
-	child->impl = impl;
-	child->view = view;
-	child->surface = surface;
-	wl_list_init(&child->children);
-
-	wl_signal_add(&surface->events.commit, &child->surface_commit);
-	child->surface_commit.notify = view_child_handle_surface_commit;
-	wl_signal_add(&surface->events.new_subsurface,
-		&child->surface_new_subsurface);
-	child->surface_new_subsurface.notify =
-		view_child_handle_surface_new_subsurface;
-	wl_signal_add(&surface->events.destroy, &child->surface_destroy);
-	child->surface_destroy.notify = view_child_handle_surface_destroy;
-
-	// Not all child views have a map/unmap event
-	child->surface_map.notify = view_child_handle_surface_map;
-	wl_list_init(&child->surface_map.link);
-	child->surface_unmap.notify = view_child_handle_surface_unmap;
-	wl_list_init(&child->surface_unmap.link);
-
-	wl_signal_add(&view->events.unmap, &child->view_unmap);
-	child->view_unmap.notify = view_child_handle_view_unmap;
-
-	struct sway_container *container = child->view->container;
-	if (container != NULL) {
-		struct sway_workspace *workspace = container->pending.workspace;
-		if (workspace) {
-			surface_enter_output(child->surface, workspace->output);
-		}
-	}
-
-	view_child_init_subsurfaces(child, surface);
-}
-
-void view_child_destroy(struct sway_view_child *child) {
-	if (view_child_is_mapped(child) && child->view->container != NULL) {
-		view_child_damage(child, true);
-	}
-
-	if (child->parent != NULL) {
-		wl_list_remove(&child->link);
-		child->parent = NULL;
-	}
-
-	struct sway_view_child *subchild, *tmpchild;
-	wl_list_for_each_safe(subchild, tmpchild, &child->children, link) {
-		wl_list_remove(&subchild->link);
-		subchild->parent = NULL;
-		// The subchild lost its parent link, so it cannot see that the parent
-		// is unmapped. Unmap it directly.
-		subchild->mapped = false;
-	}
-
-	wl_list_remove(&child->surface_commit.link);
-	wl_list_remove(&child->surface_destroy.link);
-	wl_list_remove(&child->surface_map.link);
-	wl_list_remove(&child->surface_unmap.link);
-	wl_list_remove(&child->view_unmap.link);
-	wl_list_remove(&child->surface_new_subsurface.link);
-
-	if (child->impl && child->impl->destroy) {
-		child->impl->destroy(child);
-	} else {
-		free(child);
+		wlr_scene_subsurface_tree_set_clip(&con->view->content_tree->node, &clip);
 	}
 }
 
@@ -1203,7 +968,7 @@ struct sway_view *view_from_wlr_surface(struct wlr_surface *wlr_surface) {
 	if ((xdg_surface = wlr_xdg_surface_try_from_wlr_surface(wlr_surface))) {
 		return view_from_wlr_xdg_surface(xdg_surface);
 	}
-#if HAVE_XWAYLAND
+#if WLR_HAS_XWAYLAND
 	struct wlr_xwayland_surface *xsurface;
 	if ((xsurface = wlr_xwayland_surface_try_from_wlr_surface(wlr_surface))) {
 		return view_from_wlr_xwayland_surface(xsurface);
@@ -1223,75 +988,16 @@ struct sway_view *view_from_wlr_surface(struct wlr_surface *wlr_surface) {
 	return NULL;
 }
 
-static char *escape_pango_markup(const char *buffer) {
-	size_t length = escape_markup_text(buffer, NULL);
-	char *escaped_title = calloc(length + 1, sizeof(char));
-	escape_markup_text(buffer, escaped_title);
-	return escaped_title;
-}
+void view_update_app_id(struct sway_view *view) {
+	const char *app_id = view_get_app_id(view);
 
-static size_t append_prop(char *buffer, const char *value) {
-	if (!value) {
-		return 0;
-	}
-	// If using pango_markup in font, we need to escape all markup chars
-	// from values to make sure tags are not inserted by clients
-	if (config->pango_markup) {
-		char *escaped_value = escape_pango_markup(value);
-		lenient_strcat(buffer, escaped_value);
-		size_t len = strlen(escaped_value);
-		free(escaped_value);
-		return len;
-	} else {
-		lenient_strcat(buffer, value);
-		return strlen(value);
-	}
-}
-
-/**
- * Calculate and return the length of the formatted title.
- * If buffer is not NULL, also populate the buffer with the formatted title.
- */
-static size_t parse_title_format(struct sway_view *view, char *buffer) {
-	if (!view->title_format || strcmp(view->title_format, "%title") == 0) {
-		return append_prop(buffer, view_get_title(view));
+	if (view->foreign_toplevel && app_id) {
+		wlr_foreign_toplevel_handle_v1_set_app_id(view->foreign_toplevel, app_id);
 	}
 
-	size_t len = 0;
-	char *format = view->title_format;
-	char *next = strchr(format, '%');
-	while (next) {
-		// Copy everything up to the %
-		lenient_strncat(buffer, format, next - format);
-		len += next - format;
-		format = next;
-
-		if (strncmp(next, "%title", 6) == 0) {
-			len += append_prop(buffer, view_get_title(view));
-			format += 6;
-		} else if (strncmp(next, "%app_id", 7) == 0) {
-			len += append_prop(buffer, view_get_app_id(view));
-			format += 7;
-		} else if (strncmp(next, "%class", 6) == 0) {
-			len += append_prop(buffer, view_get_class(view));
-			format += 6;
-		} else if (strncmp(next, "%instance", 9) == 0) {
-			len += append_prop(buffer, view_get_instance(view));
-			format += 9;
-		} else if (strncmp(next, "%shell", 6) == 0) {
-			len += append_prop(buffer, view_get_shell(view));
-			format += 6;
-		} else {
-			lenient_strcat(buffer, "%");
-			++format;
-			++len;
-		}
-		next = strchr(format, '%');
+	if (view->ext_foreign_toplevel) {
+		update_ext_foreign_toplevel(view);
 	}
-	lenient_strcat(buffer, format);
-	len += strlen(format);
-
-	return len;
 }
 
 void view_update_title(struct sway_view *view, bool force) {
@@ -1310,7 +1016,7 @@ void view_update_title(struct sway_view *view, bool force) {
 	free(view->container->title);
 	free(view->container->formatted_title);
 
-	size_t len = parse_title_format(view, NULL);
+	size_t len = parse_title_format(view->container, NULL);
 
 	if (len) {
 		char *buffer = calloc(len + 1, sizeof(char));
@@ -1318,7 +1024,7 @@ void view_update_title(struct sway_view *view, bool force) {
 			return;
 		}
 
-		parse_title_format(view, buffer);
+		parse_title_format(view->container, buffer);
 		view->container->formatted_title = buffer;
 	} else {
 		view->container->formatted_title = NULL;
@@ -1327,13 +1033,23 @@ void view_update_title(struct sway_view *view, bool force) {
 	view->container->title = title ? strdup(title) : NULL;
 
 	// Update title after the global font height is updated
-	container_update_title_textures(view->container);
+	if (view->container->title_bar.title_text && len) {
+		sway_text_node_set_text(view->container->title_bar.title_text,
+			view->container->formatted_title);
+		container_arrange_title_bar(view->container);
+	} else {
+		container_update_title_bar(view->container);
+	}
 
 	ipc_event_window(view->container, "title");
 
 	if (view->foreign_toplevel && title) {
 		wlr_foreign_toplevel_handle_v1_set_title(view->foreign_toplevel, title);
 	}
+
+	if (view->ext_foreign_toplevel) {
+		update_ext_foreign_toplevel(view);
+	}
 }
 
 bool view_is_visible(struct sway_view *view) {
@@ -1394,6 +1110,7 @@ void view_set_urgent(struct sway_view *view, bool enable) {
 			return;
 		}
 		clock_gettime(CLOCK_MONOTONIC, &view->urgent);
+		container_update_itself_and_parents(view->container);
 	} else {
 		view->urgent = (struct timespec){ 0 };
 		if (view->urgent_timer) {
@@ -1401,11 +1118,10 @@ void view_set_urgent(struct sway_view *view, bool enable) {
 			view->urgent_timer = NULL;
 		}
 	}
-	container_damage_whole(view->container);
 
 	ipc_event_window(view->container, "urgent");
 
-	if (!container_is_scratchpad_hidden(view->container)) {
+	if (!container_is_scratchpad_hidden_or_child(view->container)) {
 		workspace_detect_urgent(view->container->pending.workspace);
 	}
 }
@@ -1415,40 +1131,54 @@ bool view_is_urgent(struct sway_view *view) {
 }
 
 void view_remove_saved_buffer(struct sway_view *view) {
-	if (!sway_assert(!wl_list_empty(&view->saved_buffers), "Expected a saved buffer")) {
+	if (!sway_assert(view->saved_surface_tree, "Expected a saved buffer")) {
 		return;
 	}
-	struct sway_saved_buffer *saved_buf, *tmp;
-	wl_list_for_each_safe(saved_buf, tmp, &view->saved_buffers, link) {
-		wlr_buffer_unlock(&saved_buf->buffer->base);
-		wl_list_remove(&saved_buf->link);
-		free(saved_buf);
-	}
+
+	wlr_scene_node_destroy(&view->saved_surface_tree->node);
+	view->saved_surface_tree = NULL;
+	wlr_scene_node_set_enabled(&view->content_tree->node, true);
 }
 
-static void view_save_buffer_iterator(struct wlr_surface *surface,
+static void view_save_buffer_iterator(struct wlr_scene_buffer *buffer,
 		int sx, int sy, void *data) {
-	struct sway_view *view = data;
+	struct wlr_scene_tree *tree = data;
 
-	if (surface && surface->buffer) {
-		wlr_buffer_lock(&surface->buffer->base);
-		struct sway_saved_buffer *saved_buffer = calloc(1, sizeof(struct sway_saved_buffer));
-		saved_buffer->buffer = surface->buffer;
-		saved_buffer->width = surface->current.width;
-		saved_buffer->height = surface->current.height;
-		saved_buffer->x = view->container->surface_x + sx;
-		saved_buffer->y = view->container->surface_y + sy;
-		saved_buffer->transform = surface->current.transform;
-		wlr_surface_get_buffer_source_box(surface, &saved_buffer->source_box);
-		wl_list_insert(view->saved_buffers.prev, &saved_buffer->link);
+	struct wlr_scene_buffer *sbuf = wlr_scene_buffer_create(tree, NULL);
+	if (!sbuf) {
+		sway_log(SWAY_ERROR, "Could not allocate a scene buffer when saving a surface");
+		return;
 	}
+
+	wlr_scene_buffer_set_dest_size(sbuf,
+		buffer->dst_width, buffer->dst_height);
+	wlr_scene_buffer_set_opaque_region(sbuf, &buffer->opaque_region);
+	wlr_scene_buffer_set_source_box(sbuf, &buffer->src_box);
+	wlr_scene_node_set_position(&sbuf->node, sx, sy);
+	wlr_scene_buffer_set_transform(sbuf, buffer->transform);
+	wlr_scene_buffer_set_buffer(sbuf, buffer->buffer);
 }
 
 void view_save_buffer(struct sway_view *view) {
-	if (!sway_assert(wl_list_empty(&view->saved_buffers), "Didn't expect saved buffer")) {
+	if (!sway_assert(!view->saved_surface_tree, "Didn't expect saved buffer")) {
 		view_remove_saved_buffer(view);
 	}
-	view_for_each_surface(view, view_save_buffer_iterator, view);
+
+	view->saved_surface_tree = wlr_scene_tree_create(view->scene_tree);
+	if (!view->saved_surface_tree) {
+		sway_log(SWAY_ERROR, "Could not allocate a scene tree node when saving a surface");
+		return;
+	}
+
+	// Enable and disable the saved surface tree like so to atomitaclly update
+	// the tree. This will prevent over damaging or other weirdness.
+	wlr_scene_node_set_enabled(&view->saved_surface_tree->node, false);
+
+	wlr_scene_node_for_each_buffer(&view->content_tree->node,
+		view_save_buffer_iterator, view->saved_surface_tree);
+
+	wlr_scene_node_set_enabled(&view->content_tree->node, false);
+	wlr_scene_node_set_enabled(&view->saved_surface_tree->node, true);
 }
 
 bool view_is_transient_for(struct sway_view *child,
@@ -1456,3 +1186,32 @@ bool view_is_transient_for(struct sway_view *child,
 	return child->impl->is_transient_for &&
 		child->impl->is_transient_for(child, ancestor);
 }
+
+bool view_can_tear(struct sway_view *view) {
+	switch (view->tearing_mode) {
+	case TEARING_OVERRIDE_FALSE:
+		return false;
+	case TEARING_OVERRIDE_TRUE:
+		return true;
+	case TEARING_WINDOW_HINT:
+		return view->tearing_hint ==
+			WP_TEARING_CONTROL_V1_PRESENTATION_HINT_ASYNC;
+	}
+	return false;
+}
+
+static void send_frame_done_iterator(struct wlr_scene_buffer *scene_buffer,
+		int x, int y, void *data) {
+	struct timespec *when = data;
+	wl_signal_emit_mutable(&scene_buffer->events.frame_done, when);
+}
+
+void view_send_frame_done(struct sway_view *view) {
+	struct timespec when;
+	clock_gettime(CLOCK_MONOTONIC, &when);
+
+	struct wlr_scene_node *node;
+	wl_list_for_each(node, &view->content_tree->children, link) {
+		wlr_scene_node_for_each_buffer(node, send_frame_done_iterator, &when);
+	}
+}
